﻿using System.Collections.Generic;
using System.IO;
using System.Windows.Media.Imaging;
using FluxJpeg.Core;
using FluxJpeg.Core.Encoder;
using Telerik.Windows.Media.Imaging;
using Telerik.Windows.Media.Imaging.FormatProviders;

namespace CustomFormatProvider
{
    public class CustomJpegFormatProvider : IImageFormatProvider
    {
        private static readonly string[] SUPPORTED_EXTENSIONS = new string[] { ".jpg", ".jpeg" };

        // Used for the import operation, which is already supported
        private JpegFormatProvider internalJpegFormatProvider;

        public int Quality { get; set; }

        public CustomJpegFormatProvider()
        {
            this.internalJpegFormatProvider = new JpegFormatProvider();
            this.Quality = 80;
        }

        private void Encode(WriteableBitmap bitmap, Stream stream)
        {
            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;

            byte[][,] raster = new byte[3][,];

            for (int i = 0; i < 3; i++)
            {
                raster[i] = new byte[width, height];
            }

            for (int row = 0; row < height; row++)
            {
                for (int column = 0; column < width; column++)
                {
                    int pixel = bitmap.Pixels[width * row + column];
                    raster[0][column, row] = (byte)(pixel >> 16);
                    raster[1][column, row] = (byte)(pixel >> 8);
                    raster[2][column, row] = (byte)pixel;
                }
            }

            FluxJpeg.Core.Image img = new FluxJpeg.Core.Image(new ColorModel() { colorspace = ColorSpace.RGB }, raster);
            JpegEncoder encoder = new JpegEncoder(img, this.Quality, stream);
            encoder.Encode();
        }


        public bool CanExport
        {
            get { return true; }
        }

        public bool CanImport
        {
            get { return true; }
        }

        public byte[] Export(RadBitmap image)
        {
            using (MemoryStream stream = new MemoryStream())
            {
                this.Encode(image.Bitmap, stream);
                return stream.ToArray();
            }
        }

        public void Export(RadBitmap image, Stream stream)
        {
            this.Encode(image.Bitmap, stream);
        }

        public RadBitmap Import(byte[] bytes)
        {
            return this.internalJpegFormatProvider.Import(bytes);
        }

        public RadBitmap Import(Stream stream)
        {
            return this.internalJpegFormatProvider.Import(stream);
        }

        public IEnumerable<string> SupportedExtensions
        {
            get { return SUPPORTED_EXTENSIONS; }
        }
    }
}
